#!/usr/bin/env python
#coding: utf-8
#
# (c) 2007-2008 Sednacom <http://www.sednacom.fr>
#
# authors :
#  - Brice < brice@sednacom.fr >
#  - Gael < gael@sednacom.fr >

from osv import osv, fields
import pooler
import mx.DateTime as mdt

def _models_get(self, cr, uid, context={}):
    pooloo = pooler.get_pool(cr.dbname)
    obj = pooloo.get('ir.model')
    ids = obj.search(cr, uid, [('alert_allowed','=',1),])
    res = []
    for o in obj.browse(cr, uid, ids, context=context):
        res.append( [o.model, o.name] )

    res2 = [ [ el[1], el[0] ] for el in res ]
    res2.sort()
    res = [ [ el[1], el[0] ] for el in res2 ]

    return res

def _default_date(*args):
    res = mdt.today() + mdt.oneDay
    return res.strftime('%Y-%m-%d 09:00:00')

class sednacom_alert(osv.osv):
    _name = 'sednacom.alert'
    _description = 'Alerts'
    _columns = {
        'name' : fields.char('Description',required=True,size=64, states={'done' : [('readonly', True),]}) ,
        'datetime' : fields.datetime('Date/Time', required=True, states={'done' : [('readonly', True),]}) ,
        'user_to' : fields.many2one('res.users','To' ,  required=True, states={'done' : [('readonly', True),]}) ,
        'user_from' : fields.many2one('res.users','From', required=True, states={'done' : [('readonly', True),]}) ,
        'state' : fields.selection ([('pending','Pending'),('done','Done'),],'State', readonly=True) ,
        'resource' : fields.reference('Resource', selection=_models_get, size=128, states={'done' : [('readonly', True),]}) ,
        'message' : fields.text('Message', states={'done' : [('readonly', True),]}) ,
        'by_mail' : fields.boolean('Alert by mail', states={'done' : [('readonly', True),]}) ,
        'by_request' : fields.boolean('Alert by request', states={'done' : [('readonly', True),]}) ,

    }
    _order = 'datetime asc'
    _defaults = {
        'user_to' : lambda *args : args[2] ,
        'user_from' : lambda *args : args[2] ,
        'state' : lambda *args : 'pending' ,
        'by_mail' : lambda *args : True ,
        'datetime' : _default_date ,
    }

    def set_to_done(self, cr, uid, alerts, *args):
        self.write(cr, uid, alerts, {'state' : 'done'})
        return True

    def set_to_pending(self, cr, uid, alerts, *args):
        self.write(cr, uid, alerts, {'state' : 'pending'})
        return True

    def send_alert(self, cr, uid, alert, *args):
        alerts = self.browse(cr, uid, alert)
        for alert in alerts:
            by_mail = alert.by_mail
            by_request = alert.by_request
            cron_obj = self.pool.get('ir.cron')
            if by_request:
                data = {
                        'name' : "Request : "+alert.name,
                        'user_id' : uid,
                        'nextcall' : alert.datetime,
                        'model' : 'sednacom.alert',
                        'function' : 'send_request',
                        'args' : str([alert.id]),
                        }
                cron_obj.create(cr, uid, data)
            if by_mail:
                data = {
                        'name' : "Email : "+alert.name,
                        'user_id' : uid,
                        'nextcall' : alert.datetime,
                        'model' : 'sednacom.alert',
                        'function' : 'send_mail',
                        'args' : str([alert.id]),
                        }
                cron_obj.create(cr, uid, data)
        return True

    def send_request(self, cr, uid, alerts, *args):
        request_obj = self.pool.get('res.request')
        alert = self.browse(cr, uid, [alerts])[0]
        partner = False
        reference = False
        if alert.resource:
            resource = alert.resource.split(',')
            if 'res.partner' in resource:
                partner = resource[1]
            else :
                reference = alert.resource
        data = {
                    'name' : alert.name,
                    'active' : True,
                    'priority' : 1,
                    'act_from' : alert.user_from.id,
                    'act_to' : alert.user_to.id,
                    'body' : alert.message,
                    'state' : 'waiting',
                    'ref_partner_id' : partner,
                    'ref_doc1' : reference,
                    'trigger_date' : alert.datetime
                    }
        request_obj.create(cr, uid, data)
        self.set_to_done(cr, uid, [alert.id])
        return True

    def send_mail(self, cr, uid, alert, *args):
        sednaemail_obj = self.pool.get('sednacom.email')
        alert = self.browse(cr, uid, [alert])[0]
        data = {
                'name' : alert.name,
                'exp' : alert.user_from.address_id.email,
                'recipients' :  [(6, 0, [alert.user_to.address_id.id])],
                'body' : alert.message,
                }
        mail = sednaemail_obj.create(cr, uid, data)
        sednaemail_obj.send(cr, uid, [mail])
        self.set_to_done(cr, uid, [alert.id])
        return True

sednacom_alert()

