# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 HASA SARL. (http://www.hasa.ch) All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

import wizard
import pooler

class wizard_account_chart_advanced(wizard.interface):
    """wizard that display an advanced accounts chart"""
    _account_chart_arch = '''<?xml version="1.0"?>
    <form string="Account charts advanced">
        <separator string="Fields appear in order of priority" colspan="4"/>
        <field name="fiscalyear" colspan="4"/>
        <field name="periods" colspan="4"/>
        <field name="date_from"/>
        <field name="date_to"/>
        <field name="target_move"/>
    </form>'''

    _account_chart_fields = {
            'fiscalyear': {'string': 'Fiscal year','type': 'many2one', 'relation': 'account.fiscalyear',},
            'periods': {'string': 'Periods', 'type': 'many2many', 'relation': 'account.period',},
            'date_from': {'string':"Start date",'type':'date',},
            'date_to': {'string':"End date",'type':'date',},
            'target_move': {'string': 'Target Moves','type': 'selection','selection': [('all','All Entries'),('posted_only','All Posted Entries')],'required': True,'default': lambda *a:"all",},
    }

    def _get_defaults(self, cr, uid, data, context):
        """get the chosen fiscalyear/periods/dates"""
        if not(data['form']['fiscalyear'] or data['form']['periods'][0][2] or \
            (data['form']['date_from'] and data['form']['date_to'])):
            raise wizard.except_wizard('Warning !', 'You must define a year, periods or dates to continue.')
        return 'suite'

    def _account_chart_open_window(self, cr, uid, data, context):
        mod_obj = pooler.get_pool(cr.dbname).get('ir.model.data')
        act_obj = pooler.get_pool(cr.dbname).get('ir.actions.act_window')
        result = mod_obj._get_id(cr, uid, 'account', 'action_account_tree')
        ident = mod_obj.read(cr, uid, [result], ['res_id'])[0]['res_id']
        result = act_obj.read(cr, uid, [ident])[0]
        if data['form']['fiscalyear']:
            result['context'] = str({'fiscalyear': data['form']['fiscalyear'], \
            'target_move':data['form']['target_move']})
            result['name'] += ':'+str(pooler.get_pool(cr.dbname).get('account.fiscalyear').read(cr, uid, [data['form']['fiscalyear']])[0]['code'])
            return result
        if data['form']['periods'][0][2]:
            result['context'] = str({'periods': data['form']['periods'][0][2], \
            'target_move':data['form']['target_move']})
            for x in data['form']['periods'][0][2]:
                result['name'] += ':'+str(pooler.get_pool(cr.dbname).get('account.period').read(cr, uid, [x])[0]['code'])
            return result
        if data['form']['date_from'] and data['form']['date_to']:
            result['context'] = str({'date_from': data['form']['date_from'], \
            'date_to': data['form']['date_to'], 'target_move': data['form']['target_move']})
            result['name'] += ':'+str(data['form']['date_from'])+'/'+\
            str(data['form']['date_to'])
            return result

    states = {
        'init': {
            'actions': [],
            'result':{'type': 'form', 'arch':_account_chart_arch, \
            'fields':_account_chart_fields, 'state': [('end', 'Cancel'), ('open', 'Open Charts')]}
        },
        'open': {
            'actions': [],
            'result': {'type': 'action', 'action':_get_defaults, 'state':'suite'}
        },
        'suite': {
            'actions': [],
            'result': {'type': 'action', 'action':_account_chart_open_window, 'state':'end'}
        }
    }
wizard_account_chart_advanced('account.chart_advanced')
