# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) Hasa Sarl - http://www.hasa.ch All Rights Reserved.
# Author: Andre Kurz <andre.kurz@hasa.ch>
#
#    This file is part of the hasa_multi_company_cost_price module
#
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

from osv import osv, fields
import pooler

class product_multicostprice(osv.osv):
    """new class that make the relation between a product, \
    a partner and a cost price"""
    _name = "product.multicostprice"
    
    _columns = {
        'product_id': fields.many2one('product.product', 'Product', required=True),
        'partner_id': fields.many2one('res.partner', 'Partner', required=True),
        'standard_price': fields.float('Standard price'),
    }
    _order = 'product_id asc, partner_id asc'
    _sql_constraints = [
        ('partner_product_unique','unique (partner_id, product_id)','Cannot have two cost price for the same product in the same company !'),
    ]
    
product_multicostprice()

class product_product(osv.osv):
    _name = "product.product"
    _inherit = "product.product"
    
    def _get_cost_price(self, cr, uid, ids, field_name, arg, context):
        """get the cost price from product.multicostprice"""
        res = {}
        data = {}
        bool = True
        pool = pooler.get_pool(cr.dbname)
        user = pool.get('res.users').browse(cr, uid, uid)
        for id in ids:
            product_multicostprice_id = pool.get('product.multicostprice').search(cr, uid, [('partner_id', '=', user.company_id.partner_id.id), ('product_id', '=', id)])
            #if price already defined for the product we get it
            if len(product_multicostprice_id) == 1:
                product_multicostprice_obj = pool.get('product.multicostprice').browse(cr, uid, product_multicostprice_id[0])
                res[id] = product_multicostprice_obj.standard_price
            #anyelse we create it
            elif len(product_multicostprice_id) == 0:
                data['partner_id'] = pool.get('res.partner').search(cr, uid, [('id', '=', user.company_id.partner_id.id)])[0]
                try:
                    data['product_id'] = pool.get('product.product').search(cr, uid, [('id', '=', id)])[0]
                except:
                    bool = False
                    print 'ERROR: hasa_multi_company_cost_price, error product id: ', id
                data['standard_price'] = 0
                if bool:
                    if data['partner_id'] and data['product_id']:
                        self.pool.get('product.multicostprice').create(cr, uid, data)
                res[id] = 0.0
            else:
                print 'ERROR: hasa_multi_company_cost_price, cost price is defined twice for this product and for your company.'
                res[id] = 0.0
        return res

    _columns = {
        'standard_price': fields.function(_get_cost_price,method=True, readonly=False, type='float', string='Standard cost price'),
    }
    
product_product()

class stock_location(osv.osv):
    _name = "stock.location"
    _inherit = "stock.location"

    _columns = {
        'company_id': fields.many2one('res.company', 'Company (cost price)',),
    }
    
stock_location()