# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import fields
from osv import osv
import ir

import netsvc
import time
from mx import DateTime
from tools.translate import _
import math


def name_serial(x, y, z, p, f, d):
     lote = '_'
     if (f == 'quadrangular'):
         lote = (str(x) + 'x' + str(y) + 'x' + str(z) + '_' + str(p))
     elif (f == 'cylindrical'):
         lote = (str(d) + 'x' + str(z) + '_' + str(p))
     return lote

def compute_w(size_x, size_y, size_z, shape, density, diameter):
       resu = {}

       weight = 0
       if shape == 'quadrangular':
           weight = ((size_x * size_y * size_z) * density) / 1000000
           diameter = 0
       elif shape == 'cylindrical':
               radius = diameter / 2.0
               weight = ((math.pow(radius, 2) * math.pi * size_z) * density) / 1000000
               size_x = 0
               size_y = 0
       else:
               weight = 0
               size_x = 0
               size_y = 0
               size_z = 0
               density = 0

       v = { 'weight':weight,
             'size_x':size_x,
             'size_y':size_y,
             'size_z':size_z,
             'diameter':diameter,
             'density':density,
             }
       return v


class product_product(osv.osv):
    _inherit = 'product.product'
    _columns = {
        'size_x': fields.float('Width'),
        'size_y': fields.float('Length'),
        'size_z': fields.float('Thickness'),
        'density': fields.float('Density'),
        'shape' : fields.selection([('quadrangular', 'Quadrangular'), ('cylindrical', 'Cylindrical'), ('other', 'Other')], 'Shape', required=True),
        'diameter' : fields.float('Diameter'),
        'purchase_price': fields.selection((('weight','Weight'),('units','Units')),'Purchase price in',required=True),
    }
    _defaults = {
        'diameter': lambda * a: 0.0,
        'shape': lambda * a: 'other',
        'size_x': lambda * a: 0.0,
        'size_y': lambda * a: 0.0,
        'size_z': lambda * a: 0.0,
        'density': lambda * a: 0.0,
        'purchase_price': lambda * a: 'units',
        }

    def compute_weight(self, cr, uid, id, size_x, size_y, size_z, shape, density, diameter):
       return {'value': compute_w(size_x, size_y, size_z, shape, density, diameter)}

product_product()


class stock_production_lot(osv.osv):
    _name = 'stock.production.lot'
    _inherit = 'stock.production.lot'
    _columns = {
        'size_x': fields.float('Width'),
        'size_y': fields.float('Length'),
        'size_z': fields.float('Thickness'),
        'shape' : fields.selection([('quadrangular', 'Quadrangular'), ('cylindrical', 'Cylindrical'), ('other', 'Other')], 'Shape', required=True),
        'diameter' : fields.float('Diameter'),
        'weight': fields.float('Weight'),
        'density': fields.float('Density'),
    }
    _defaults = {
        'diameter': lambda * a: 0.0,
        'shape': lambda * a: 'other',
        'size_x': lambda * a: 0.0,
        'size_y': lambda * a: 0.0,
        'size_z': lambda * a: 0.0,
        'weight': lambda * a: 0.0,
    }

    def generate_serial(self, cr, uid, ids, context={}):
       radio = 0
       for w in self.browse(cr, uid, ids):
           v = {
                'name' : name_serial(w.size_x, w.size_y, w.size_z, w.weight, w.shape, w.diameter)
                }
           w.write(v)

    def compute_weight(self, cr, uid, id, size_x, size_y, size_z, shape, density, diameter):
       return {'value': compute_w(size_x, size_y, size_z, shape, density, diameter)}

stock_production_lot()


class mrp_production_product_line(osv.osv):
    _name = 'mrp.production.product.line'
    _inherit = 'mrp.production.product.line'
    _columns = {
        'size_x': fields.float('Width'),
        'size_y': fields.float('Length'),
        'size_z': fields.float('Thickness'),
        'density': fields.float('Density'),
        'shape' : fields.selection([('quadrangular', 'Quadrangular'), ('cylindrical', 'Cylindrical'), ('other', 'Other')], 'Shape', required=True),
        'diameter' : fields.float('Diameter'),
        'weight': fields.float('Weight')
        }
    _defaults = {
        'shape': lambda * a: 'other',
        }

    def onchange_product_id(self, cr, uid, ids, product_id, context={}):
        if product_id:
            w = self.pool.get('product.product').browse(cr, uid, product_id, context)
            v = {
                'product_uom':w.uom_id.id,
                'product_uos':w.uos_id and w.uos_id.id or w.uom_id.id,
                'size_x':w.size_x,
                'size_y':w.size_y,
                'size_z':w.size_z,
                'density':w.density,
                'shape':w.shape,
                'diameter':w.diameter,
                'weight':compute_w(w.size_x, w.size_y, w.size_z, w.shape, w.density, w.diameter)['weight']
            }
            return {'value': v}
        return {}

    def compute_weight(self, cr, uid, id, size_x, size_y, size_z, shape, density, diameter):
       return {'value': compute_w(size_x, size_y, size_z, shape, density, diameter)}

mrp_production_product_line()


class mrp_production(osv.osv):
    _name = 'mrp.production'
    _description = 'Production'
    _inherit = 'mrp.production'
    _columns = {
        'product_line_origin': fields.many2one('mrp.production.product.line', 'Product line origin'),
    }

    def action_compute(self, cr, uid, ids, properties=[]):
        results = []
        for production in self.browse(cr, uid, ids):
            cr.execute('delete from mrp_production_product_line where production_id=%s', (production.id,))
            cr.execute('delete from mrp_production_workcenter_line where production_id=%s', (production.id,))
            bom_point = production.bom_id
            bom_id = production.bom_id.id
            if not bom_point:
                bom_id = self.pool.get('mrp.bom')._bom_find(cr, uid, production.product_id.id, production.product_uom.id, properties)
                if bom_id:
                    bom_point = self.pool.get('mrp.bom').browse(cr, uid, bom_id)
                    routing_id = bom_point.routing_id.id or False
                    self.write(cr, uid, [production.id], {'bom_id': bom_id, 'routing_id': routing_id})

            if not bom_id:
                raise osv.except_osv(_('Error'), _("Couldn't find bill of material for product"))

            factor = production.product_qty * production.product_uom.factor / bom_point.product_uom.factor
            res = self.pool.get('mrp.bom')._bom_explode(cr, uid, bom_point, factor / bom_point.product_qty, properties)
            results = res[0]
            results2 = res[1]
            i = 0
            for line in results:
                line['production_id'] = production.id
                a = self.pool.get('mrp.production.product.line').create(cr, uid, line)
                production_line = self.pool.get('mrp.production.product.line').browse(cr, uid, a)
                w = False
                w2 = False
                if not (production.product_line_origin):
                    p = self.pool.get('mrp.bom').browse(cr, uid, bom_id)
                    w = self.pool.get('mrp.bom').browse(cr, uid, p.bom_lines[i].id)
                    if w:
                        v = {
                            'product_uom':w.product_uom.id,
                            'product_uos':w.product_uos and w.product_uos.id or w.product_uom.id,
                            'size_x':w.size_x,
                            'size_y':w.size_y,
                            'size_z':w.size_z,
                            'density':w.density,
                            'shape':w.shape,
                            'diameter':w.diameter,
                            'weight':w.weight
                            }
                else:
                    origen = production.origin[1:]
#                    production2 = self.pool.get('mrp.production').search(cr, uid, [('name', '=', origen)])[0]

                    w2 = self.pool.get('mrp.production.product.line').browse(cr, uid, production.product_line_origin.id)
                    v = {
                        'size_x':w2.size_x,
                        'size_y':w2.size_y,
                        'size_z':w2.size_z,
                        'density':w2.density,
                        'shape':w2.shape,
                        'diameter':w2.diameter,
                        'weight':w2.weight
                    }
                if (w or w2):
                    production_line.write(v)
                i = i + 1
            for line in results2:
                line['production_id'] = production.id
                self.pool.get('mrp.production.workcenter.line').create(cr, uid, line)

        return len(results)

    def action_confirm(self, cr, uid, ids):
        picking_id = False
        lote_id = False
        proc_ids = []
        for production in self.browse(cr, uid, ids):
            if not production.product_lines:
                self.action_compute(cr, uid, [production.id])
                production = self.browse(cr, uid, [production.id])[0]
            routing_loc = None
            pick_type = 'internal'
            address_id = False
            if production.bom_id.routing_id and production.bom_id.routing_id.location_id:
                routing_loc = production.bom_id.routing_id.location_id
                if routing_loc.usage <> 'internal':
                    pick_type = 'out'
                address_id = routing_loc.address_id and routing_loc.address_id.id or False
                routing_loc = routing_loc.id
            picking_id = self.pool.get('stock.picking').create(cr, uid, {
                'origin': (production.origin or '').split(':')[0] + ':' + production.name,
                'type': pick_type,
                'move_type': 'one',
                'state': 'auto',
                'address_id': address_id,
                'auto_picking': self._get_auto_picking(cr, uid, production),
            })

            if (production.product_line_origin and production.product_line_origin.id):
                product_line_orig = self.pool.get('mrp.production.product.line').browse(cr, uid, production.product_line_origin.id)

                name_seriale = name_serial(product_line_orig.size_x, product_line_orig.size_y, product_line_orig.size_z, product_line_orig.weight, product_line_orig.shape, product_line_orig.diameter)
                l = self.pool.get('stock.production.lot').search(cr, uid, [('product_id', '=', product_line_orig.product_id.id), ('name', '=', name_seriale)])
                if not l:
                        datalote = {
                            'name':name_seriale,
                            'product_id': product_line_orig.product_id.id,
                            'date' : production.date_planned
                            }
                        lote_id = self.pool.get('stock.production.lot').create(cr, uid, datalote)
                else:
                    lote_id = l[0]

            source = production.product_id.product_tmpl_id.property_stock_production.id
            data = {
                'name':'PROD:' + production.name,
                'date_planned': production.date_planned,
                'product_id': production.product_id.id,
                'product_qty': production.product_qty,
                'product_uom': production.product_uom.id,
                'product_uos_qty': production.product_uos and production.product_uos_qty or False,
                'product_uos': production.product_uos and production.product_uos.id or False,
                'location_id': source,
                'location_dest_id': production.location_dest_id.id,
                'move_dest_id': production.move_prod_id.id,
                'state': 'waiting',
                'prodlot_id': lote_id or False
            }
            res_final_id = self.pool.get('stock.move').create(cr, uid, data)

            self.write(cr, uid, [production.id], {'move_created_ids': [(6, 0, [res_final_id])]})
            moves = []
            for line in production.product_lines:

                name_seriale = name_serial(line.size_x, line.size_y, line.size_z, line.weight, line.shape, line.diameter)
                l = self.pool.get('stock.production.lot').search(cr, uid, [('product_id', '=', line.product_id.id), ('name', '=', name_seriale)])
                if not l:
                        datalote = {
                            'name':name_seriale,
                            'product_id': line.product_id.id,
                            'date' : production.date_planned,
                            'size_x' : line.size_x,
                            'size_y' : line.size_y,
                            'size_z' : line.size_z,
                            'shape' : line.shape,
                            'diameter' : line.diameter,
                            'density' : line.density,
                            'weight' : line.weight,
                            }
                        lote_id = self.pool.get('stock.production.lot').create(cr, uid, datalote)
                else:
                    lote_id = l[0]
                move_id = False
                newdate = production.date_planned
                if line.product_id.type in ('product', 'consu'):
                    res_dest_id = self.pool.get('stock.move').create(cr, uid, {
                        'name':'PROD:' + production.name,
                        'date_planned': production.date_planned,
                        'product_id': line.product_id.id,
                        'product_qty': line.product_qty,
                        'product_uom': line.product_uom.id,
                        'product_uos_qty': line.product_uos and line.product_uos_qty or False,
                        'product_uos': line.product_uos and line.product_uos.id or False,
                        'location_id': routing_loc or production.location_src_id.id,
                        'location_dest_id': source,
                        'move_dest_id': res_final_id,
                        'state': 'waiting',
                        'prodlot_id' : lote_id,
                    })
                    moves.append(res_dest_id)
                    move_id = self.pool.get('stock.move').create(cr, uid, {
                        'name':'PROD:' + production.name,
                        'picking_id':picking_id,
                        'product_id': line.product_id.id,
                        'product_qty': line.product_qty,
                        'product_uom': line.product_uom.id,
                        'product_uos_qty': line.product_uos and line.product_uos_qty or False,
                        'product_uos': line.product_uos and line.product_uos.id or False,
                        'date_planned': newdate,
                        'move_dest_id': res_dest_id,
                        'location_id': production.location_src_id.id,
                        'location_dest_id': routing_loc or production.location_src_id.id,
                        'state': 'waiting',
                        'prodlot_id' : lote_id,
                    })
                proc_id = self.pool.get('mrp.procurement').create(cr, uid, {
                    'name': (production.origin or '').split(':')[0] + ':' + production.name,
                    'origin': (production.origin or '').split(':')[0] + ':' + production.name,
                    'date_planned': newdate,
                    'product_id': line.product_id.id,
                    'product_qty': line.product_qty,
                    'product_uom': line.product_uom.id,
                    'product_uos_qty': line.product_uos and line.product_qty or False,
                    'product_uos': line.product_uos and line.product_uos.id or False,
                    'location_id': production.location_src_id.id,
                    'procure_method': line.product_id.procure_method,
                    'move_id': move_id,
                    'product_line_origin' : line.id,

                })
                wf_service = netsvc.LocalService("workflow")
                wf_service.trg_validate(uid, 'mrp.procurement', proc_id, 'button_confirm', cr)
                proc_ids.append(proc_id)
            wf_service = netsvc.LocalService("workflow")
            wf_service.trg_validate(uid, 'stock.picking', picking_id, 'button_confirm', cr)
            self.write(cr, uid, [production.id], {'picking_id':picking_id, 'move_lines': [(6, 0, moves)], 'state':'confirmed'})
        return picking_id

mrp_production()


class mrp_procurement(osv.osv):
    '''
    Method rewritting to create draft moves
    '''
    _name = 'mrp.procurement'
    _inherit = 'mrp.procurement'
    _columns = {
        'product_line_origin': fields.many2one('mrp.production.product.line', 'Production line origin'),
    }

    def action_produce_assign_product(self, cr, uid, ids, context={}):
        produce_id = False
        company = self.pool.get('res.users').browse(cr, uid, uid, context).company_id
        for procurement in self.browse(cr, uid, ids):
            res_id = procurement.move_id.id
            loc_id = procurement.location_id.id
            newdate = DateTime.strptime(procurement.date_planned, '%Y-%m-%d %H:%M:%S') - DateTime.RelativeDateTime(days=procurement.product_id.product_tmpl_id.produce_delay or 0.0)
            newdate = newdate - DateTime.RelativeDateTime(days=company.manufacturing_lead)
            produce_id = self.pool.get('mrp.production').create(cr, uid, {
                'origin': procurement.origin,
                'product_id': procurement.product_id.id,
                'product_qty': procurement.product_qty,
                'product_uom': procurement.product_uom.id,
                'product_uos_qty': procurement.product_uos and procurement.product_uos_qty or False,
                'product_uos': procurement.product_uos and procurement.product_uos.id or False,
                'location_src_id': procurement.location_id.id,
                'location_dest_id': procurement.location_id.id,
                'bom_id': procurement.bom_id and procurement.bom_id.id or False,
                'date_planned': newdate.strftime('%Y-%m-%d %H:%M:%S'),
                'move_prod_id': res_id,
                'product_line_origin' : procurement.product_line_origin and procurement.product_line_origin.id or False,
            })
            self.write(cr, uid, [procurement.id], {'state':'running'})
            bom_result = self.pool.get('mrp.production').action_compute(cr, uid,
                    [produce_id], properties=[x.id for x in procurement.property_ids])
            self.pool.get('stock.move').write(cr, uid, [res_id],
                    {'location_id':procurement.location_id.id})
        return produce_id

    def action_po_assign(self, cr, uid, ids, context={}):

        purchase_id = False
        company = self.pool.get('res.users').browse(cr, uid, uid, context).company_id
        for procurement in self.browse(cr, uid, ids):
            res_id = procurement.move_id.id
            partner = procurement.product_id.seller_ids[0].name
            partner_id = partner.id
            address_id = self.pool.get('res.partner').address_get(cr, uid, [partner_id], ['delivery'])['delivery']
            pricelist_id = partner.property_product_pricelist_purchase.id
            # Search the origin production
            production_name = procurement.origin
            production_name = production_name.split(':')
            for prod_name in production_name:
               production_id = self.pool.get('mrp.production').search(cr, uid, [('name', '=', prod_name)])
               if production_id:
                   break
            uom_id = procurement.product_id.uom_po_id.id

            qty = self.pool.get('product.uom')._compute_qty(cr, uid, procurement.product_uom.id, procurement.product_qty, uom_id)
            if procurement.product_id.seller_ids[0].qty:
                qty = max(qty, procurement.product_id.seller_ids[0].qty)

            price = self.pool.get('product.pricelist').price_get(cr, uid, [pricelist_id], procurement.product_id.id, qty, False, {'uom': uom_id})[pricelist_id]

            newdate = DateTime.strptime(procurement.date_planned, '%Y-%m-%d %H:%M:%S')
            newdate = newdate - DateTime.RelativeDateTime(days=company.po_lead)
            newdate = newdate - procurement.product_id.seller_ids[0].delay

            context.update({'lang':partner.lang})
            product = self.pool.get('product.product').browse(cr, uid, procurement.product_id.id, context=context)

            line = {
                'name': product.name,
                'product_qty': qty,
                'product_id': procurement.product_id.id,
                'product_uom': uom_id,
                'price_unit': price,
                'date_planned': newdate.strftime('%Y-%m-%d %H:%M:%S'),
                'move_dest_id': res_id,
                'notes':product.description_purchase,
            }
            if procurement.product_line_origin:
                line['size_x'] = procurement.product_line_origin.size_x
                line['size_y'] = procurement.product_line_origin.size_y
                line['size_z'] = procurement.product_line_origin.size_z
                line['density'] = procurement.product_line_origin.density
                line['diameter'] = procurement.product_line_origin.diameter
                line['weight'] = procurement.product_line_origin.weight
                line['shape'] = procurement.product_line_origin.shape
                line['purchase_price'] = procurement.product_line_origin.product_id.purchase_price

            if production_id:
                line['production_id'] = production_id[0]
            taxes_ids = procurement.product_id.product_tmpl_id.supplier_taxes_id
            taxes = self.pool.get('account.fiscal.position').map_tax(cr, uid, partner.property_account_position, taxes_ids)
            line.update({
                'taxes_id':[(6, 0, taxes)]
            })
            purchase_id = self.pool.get('purchase.order').create(cr, uid, {
                'origin': procurement.origin,
                'partner_id': partner_id,
                'partner_address_id': address_id,
                'location_id': procurement.location_id.id,
                'pricelist_id': pricelist_id,
                'order_line': [(0, 0, line)],
                'fiscal_position': partner.property_account_position and partner.property_account_position.id or False,
            })
            self.write(cr, uid, [procurement.id], {'state':'running', 'purchase_id':purchase_id})
        return purchase_id

mrp_procurement()


class purchase_order(osv.osv):
    _name = "purchase.order"
    _inherit = "purchase.order"

    def _amount_all(self, cr, uid, ids, field_name, arg, context):
        res = {}
        cur_obj=self.pool.get('res.currency')
        for order in self.browse(cr, uid, ids):
            res[order.id] = {
                'amount_untaxed': 0.0,
                'amount_tax': 0.0,
                'amount_total': 0.0,
            }
            val = val1 = 0.0
            cur=order.pricelist_id.currency_id
            for line in order.order_line:
                if line.purchase_price == 'weight':
                    product_qty = line.weight * line.product_qty
                else:
                    product_qty = line.product_qty
                for c in self.pool.get('account.tax').compute(cr, uid, line.taxes_id, line.price_unit, product_qty, order.partner_address_id.id, line.product_id, order.partner_id):
                    val+= c['amount']
                val1 += line.price_subtotal
            res[order.id]['amount_tax']=cur_obj.round(cr, uid, cur, val)
            res[order.id]['amount_untaxed']=cur_obj.round(cr, uid, cur, val1)
            res[order.id]['amount_total']=res[order.id]['amount_untaxed'] + res[order.id]['amount_tax']
        return res

    def action_picking_create(self, cr, uid, ids, *args):
        picking_id = False
        for order in self.browse(cr, uid, ids):
            loc_id = order.partner_id.property_stock_supplier.id
            istate = 'none'
            if order.invoice_method == 'picking':
                istate = '2binvoiced'
            picking_id = self.pool.get('stock.picking').create(cr, uid, {
                'origin': order.name + ((order.origin and (':' + order.origin)) or ''),
                'type': 'in',
                'address_id': order.dest_address_id.id or order.partner_address_id.id,
                'invoice_state': istate,
                'purchase_id': order.id,
            })
            for order_line in order.order_line:
                lote_id = False

                production_line_orig = self.pool.get('mrp.production.product.line').search(cr, uid,
                    [('production_id', '=', order_line.production_id.id),
                     ('product_id', '=', order_line.product_id.id),
                     ('size_x', '=', order_line.size_x),
                     ('size_y', '=', order_line.size_y),
                     ('shape', '=', order_line.shape)])
                if production_line_orig and production_line_orig[0] :
                    product_line_orig = self.pool.get('mrp.production.product.line').browse(cr, uid, production_line_orig[0])
                    name_seriale = name_serial(product_line_orig.size_x, product_line_orig.size_y, product_line_orig.size_z, product_line_orig.weight, product_line_orig.shape, product_line_orig.diameter)
                    l = self.pool.get('stock.production.lot').search(cr, uid, [('product_id', '=', product_line_orig.product_id.id), ('name', '=', name_seriale)])
                    if not l:
                            datalote = {
                                'name':name_seriale,
                                'product_id': product_line_orig.product_id.id,
                                'date' : production.date_planned,
                                'size_x' : product_line_orig.size_x,
                                'size_y' : product_line_orig.size_y,
                                'size_z' : product_line_orig.size_z,
                                'shape' : product_line_orig.shape,
                                'diameter' : product_line_orig.diameter,
                                'density' : product_line_orig.density,
                                'weight' : product_line_orig.weight
                                }
                            lote_id = self.pool.get('stock.production.lot').create(cr, uid, datalote)
                    else:
                        lote_id = l[0]

                if not order_line.product_id:
                    continue
                if order_line.product_id.product_tmpl_id.type in ('product', 'consu'):
                    dest = order.location_id.id
                    self.pool.get('stock.move').create(cr, uid, {
                        'name': 'PO:' + order_line.name,
                        'product_id': order_line.product_id.id,
                        'product_qty': order_line.product_qty,
                        'product_uos_qty': order_line.product_qty,
                        'product_uom': order_line.product_uom.id,
                        'product_uos': order_line.product_uom.id,
                        'date_planned': order_line.date_planned,
                        'location_id': loc_id,
                        'location_dest_id': dest,
                        'picking_id': picking_id,
                        'move_dest_id': order_line.move_dest_id.id,
                        'state': 'assigned',
                        'purchase_line_id': order_line.id,
                        'prodlot_id': lote_id
                    })
                    if order_line.move_dest_id:
                        self.pool.get('stock.move').write(cr, uid, [order_line.move_dest_id.id], {'location_id':order.location_id.id})
            wf_service = netsvc.LocalService("workflow")
            wf_service.trg_validate(uid, 'stock.picking', picking_id, 'button_confirm', cr)
        return picking_id

purchase_order()


class purchase_order_line(osv.osv):
    _name = "purchase.order.line"
    _inherit = "purchase.order.line"

    def _amount_line(self, cr, uid, ids, prop, unknow_none,unknow_dict):
        res = {}
        cur_obj=self.pool.get('res.currency')
        for line in self.browse(cr, uid, ids):
            cur = line.order_id.pricelist_id.currency_id
            if line.purchase_price == 'weight':
                product_qty = line.weight * line.product_qty
            else:
                product_qty = line.product_qty
            res[line.id] = cur_obj.round(cr, uid, cur, line.price_unit * product_qty)
        return res

    def compute_line_weight(self, size_x, size_y, size_z, shape, density, diameter):
        if shape == 'quadrangular':
            weight = ((size_x * size_y * size_z) * density) / 1000000
        elif shape == 'cylindrical':
            radius = diameter / 2.0
            weight = ((math.pow(radius, 2) * math.pi * size_z) * density) / 1000000
        else:
            weight = 0
        return weight

    def product_id_change(self, cr, uid, ids, pricelist, product, qty, uom,
            partner_id, date_order=False, fiscal_position=False):
        result = super(purchase_order_line, self).product_id_change(cr, uid, ids, pricelist, product, qty, uom,
            partner_id, date_order, fiscal_position)
        if not product:
            return result
        for product_obj in self.pool.get('product.product').browse(cr, uid, [product]):
            result['value']['purchase_price'] = product_obj.purchase_price
            result['value']['shape'] = product_obj.shape
            result['value']['size_x'] = product_obj.size_x
            result['value']['size_y'] = product_obj.size_y
            result['value']['size_z'] = product_obj.size_z
            result['value']['density'] = product_obj.density
            result['value']['diameter'] = product_obj.diameter
            result['value']['weight'] = self.compute_line_weight(product_obj.size_x, product_obj.size_y, product_obj.size_z, product_obj.shape, product_obj.density, product_obj.diameter)
        return result

    _columns = {
        'production_id': fields.many2one('mrp.production', 'Production', readonly=True),
        'size_x': fields.float('Width', ),
        'size_y': fields.float('Length', ),
        'size_z': fields.float('Thickness',),
        'density': fields.float('Density', ),
        'shape' : fields.selection([('quadrangular', 'Quadrangular'), ('cylindrical', 'Cylindrical'), ('other', 'Other')], 'Shape', required=True,),
        'diameter' : fields.float('Diameter', ),
        'weight': fields.float('Weight', ),
        'purchase_price': fields.selection((('weight','Weight'),('units','Units')),'Price in',required=True),
        'price_subtotal': fields.function(_amount_line, method=True, string='Subtotal'),
    }
    _defaults = {
        'shape': lambda * a: 'other',
        'purchase_price': lambda * a: 'units',
         }
    def compute_weight(self, cr, uid, id, size_x, size_y, size_z, shape, density, diameter):
       return {'value': compute_w(size_x, size_y, size_z, shape, density, diameter)}

purchase_order_line()


class mrp_bom(osv.osv):
    _name = 'mrp.bom'
    _description = 'Bill of Material'
    _inherit = 'mrp.bom'
    _columns = {
        'size_x': fields.float('Width'),
        'size_y': fields.float('Length'),
        'size_z': fields.float('Thickness'),
        'density': fields.float('Density'),
        'shape' : fields.selection([('quadrangular', 'Quadrangular'), ('cylindrical', 'Cylindrical'), ('other', 'Other')], 'Shape', required=True),
        'diameter' : fields.float('Diameter'),
        'weight': fields.float('Weight'),
        }

    def compute_weight(self, cr, uid, id, size_x, size_y, size_z, shape, density, diameter):
       return {'value': compute_w(size_x, size_y, size_z, shape, density, diameter)}

    def onchange_product_id(self, cr, uid, ids, product_id, name, id, context={}):
        if product_id:
            w = self.pool.get('product.product').browse(cr, uid, [product_id], context)[0]
            v = {
                'product_uom':w.uom_id.id,
                'product_uos':w.uos_id and w.uos_id.id or w.uom_id.id,
                'size_x':w.size_x,
                'size_y':w.size_y,
                'size_z':w.size_z,
                'density':w.density,
                'shape':w.shape,
                'diameter':w.diameter,
                'weight': compute_w(w.size_x, w.size_y, w.size_z, w.shape, w.density, w.diameter)['weight'],
                'name' : w.name,
            }
            if id :
                v['bom_id'] = id
            return {'value': v}
        return {}

mrp_bom()

