# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution    
#    Copyright (C) 2009 Smile.fr. All Rights Reserved
#    author: Raphaël Valyi
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import fields, osv


class stock_move(osv.osv):
    _inherit = "stock.move"
    
    def onchange_product_id(self, cr, uid, ids, product_id=False, location_id=False, location_dest_id=False, prodlot_id=False):
        result = super(stock_move, self).onchange_product_id(cr, uid, ids, product_id, location_id, location_dest_id)
        if not result.get('value', False):
            result.update({'value':{}})
        result['value']['new_hardware_revision_id'] = self._get_hardware_revision_id_from_ids(cr, uid, product_id, prodlot_id)
        return result
    
    def onchange_lot_id(self, cr, uid, ids, prodlot_id=False, product_qty=False, location_id=False, product_id=False):
        result = super(stock_move, self).onchange_lot_id(prodlot_id, product_qty, location_id, product_id)
        if not result.get('value', False):
            result.update({'value':{}})
        result['value']['new_hardware_revision_id'] = self._get_hardware_revision_id_from_ids(cr, uid, product_id, prodlot_id)
        return result
    
    def _get_hardware_revision_id_from_ids(self, cr, uid, product_id, prodlot_id):
        product = product_id and self.pool.get('product.product').browse(cr, uid, product_id) or False
        prodlot = prodlot_id and self.pool.get('stock.production.lot').browse(cr, uid, prodlot_id) or False
        return self._get_hardware_revision_id(cr, uid, product, prodlot)
    
    def _get_hardware_revision_id(self, cr, uid, product, prodlot):
        if prodlot and product:
            if prodlot.revisions:
                rev_name = prodlot.revisions[len(prodlot.revisions) - 1].name
                hardware_revision_ids = self.pool.get('product.hardware.revision').search(cr, uid, [('product_id','=', product.id), ('name', '=', rev_name)])
                return len(hardware_revision_ids) > 0 and hardware_revision_ids[0] or False
        elif product:
            return product.current_default_revision_id and product.current_default_revision_id.id or False

    
    def _get_new_hardware_revision_id(self, cr, uid, ids, field_name, arg, context={}):
        """Displays the default current product hardware revision if no prodlot exists, else displays selected product hardware revision
        by reading the stock.production.lot.revision items. If no item matches a product.hardware.revision, display nothing. 
        """
        res = {}
        for move in self.browse(cr, uid, ids):
            res[move.id] = self._get_hardware_revision_id(cr, uid, move.product_id, move.prodlot_id)

        return res

    
    def _set_new_hardware_revision_id(self, cr, uid, ids, name, value, arg, context):
        if not value: return False

        if isinstance(ids, (int, long)):
            ids = [ids]
            
        for move in self.browse(cr, uid, ids):
            if move.picking_id.type == "out" or move.picking_id.type == "internal":
                continue
    
            product_id = move.product_id.id
            existing_prodlot = move.prodlot_id

            hardware_revision = self.pool.get('product.hardware.revision').browse(cr, uid, value)
            if existing_prodlot:
                if not existing_prodlot.revisions:
                    revision_id = self.pool.get('stock.production.lot.revision').create(cr, uid, {
                        'name': hardware_revision.name,
                        'indice': hardware_revision.revision,
                        'lot_id': existing_prodlot.id,
                    })
                    return revision_id
                else:
                    revision = existing_prodlot.revisions[len(existing_prodlot.revisions) - 1]
                    self.pool.get('stock.production.lot.revision').write(cr, uid, revision.id, {'name': hardware_revision.name, 'indice': hardware_revision.revision})
            else:
                prodlot_id = self.pool.get('stock.production.lot').create(cr, uid, {
                    'name': "PENDING_prodlot_rev" + hardware_revision.name,
                    'product_id': product_id,
                })
                self.write(cr, uid, ids, {'prodlot_id': prodlot_id})
                revision_id = self.pool.get('stock.production.lot.revision').create(cr, uid, {
                        'name': hardware_revision.name,
                        'indice': hardware_revision.revision,
                        'lot_id': prodlot_id,
                })
                return revision_id

    _columns = {
        'new_hardware_revision_id': fields.function(_get_new_hardware_revision_id, fnct_inv=_set_new_hardware_revision_id,
                 method=True, type='many2one', relation="product.hardware.revision", size=64, string='Hardware Revision to associate', domain="[('product_id','=',product_id)]"),
    }
    
stock_move()