# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution    
#    Copyright (C) 2009 Smile.fr. All Rights Reserved
#    author: Raphaël Valyi
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import fields,osv


class stock_picking(osv.osv):
    _inherit = "stock.picking"
    
    _columns = {
            'related_loan_end_date': fields.related('sale_id', 'loan_end_date', type='date', string='Loan End Date'),
    }

stock_picking()


class stock_warehouse(osv.osv):
    _inherit = "stock.warehouse"
    _columns = {
        'property_stock_loan_inside_location': fields.property(
            'stock.location',
            type='many2one',
            relation='stock.location',
            string="Loan Inside Location",
            method=True,
            view_load=True,
            domain="[('usage', '=', 'internal')]",
            help="Loan Inside Location",
            required=True),
        'property_stock_loan_outside_location': fields.property(
            'stock.location',
            type='many2one',
            relation='stock.location',
            string="Loan Outside Location",
            method=True,
            view_load=True,
            domain="[('usage', '!=', 'internal')]",
            help="Loan OutSide Location",
            required=True),
    }
stock_warehouse()


class stock_production_lot(osv.osv):
    _inherit = "stock.production.lot"
    
    def _current_partner_loan_id(self, cr, uid, ids, field_name, arg, context={}):
        res = {}
        for prodlot_id in ids:
            cr.execute("select id from stock_move where stock_move.prodlot_id = %s and stock_move.state='done' order by stock_move.date_planned DESC LIMIT 1" % prodlot_id)
            results = cr.fetchone()
            if results and len(results) > 0:
                last_move = self.pool.get('stock.move').browse(cr, uid, results[0])
                if last_move.picking_id and last_move.picking_id.sale_id:
                    res[prodlot_id] = last_move.picking_id.sale_id.partner_id.id
                else:
                    res[prodlot_id] = False
            else:
                res[prodlot_id] = False
        return res
    
    def _current_loan_end_date(self, cr, uid, ids, field_name, arg, context={}):
        res = {}
        for prodlot_id in ids:
            cr.execute("select id from stock_move where stock_move.prodlot_id = %s and stock_move.state='done' order by stock_move.date_planned DESC LIMIT 1" % prodlot_id)
            results = cr.fetchone()
            if results and len(results) > 0:
                last_move = self.pool.get('stock.move').browse(cr, uid, results[0])
                if last_move.picking_id and last_move.picking_id.sale_id:
                    res[prodlot_id] = last_move.picking_id.sale_id.loan_end_date
                else:
                    res[prodlot_id] = False
            else:
                res[prodlot_id] = False
        return res
    
    _columns = {
            'loan_manager_id':fields.many2one('res.users', 'Loan Manager'),
            'current_partner_loan_id': fields.function(_current_partner_loan_id, method=True, type="many2one", relation="res.partner", string="Current Partner Loan"),
            'current_loan_end_date': fields.function(_current_loan_end_date, method=True, type="date", string="Current Loan End Date"),
    }

stock_production_lot()


class stock_report_prodlots(osv.osv):
    _inherit = "stock.report.prodlots"

    _columns = {
        'partner_id'    : fields.related('prodlot_id', 'current_partner_loan_id', type="many2one", relation='res.partner', readonly=True),
        'partner_name'  : fields.related('partner_id', 'name', type="char", string="Partner", readonly=True),
        'manager_id'    : fields.related('prodlot_id', 'loan_manager_id', type="many2one", relation='res.users', readonly=True),
        'manager_name'  : fields.related('manager_id', 'name', type="char", string="Loan Manager", readonly=True),
    }

stock_report_prodlots()