# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution	
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import time
from report import report_sxw
from osv import osv
import pooler

class report_contract(report_sxw.rml_parse):
    def __init__(self, cr, uid, name, context):
        super(report_contract, self).__init__(cr, uid, name, context)
        self.localcontext.update({
            'time': time,
            'get_projects': self.get_projects,
            'get_tasks': self.get_tasks,
            'get_conditions': self.get_conditions,
            'get_services': self.get_services,
            'get_project_workers': self.get_project_workers,
            'get_tasks_by_worker': self.get_tasks_by_worker,
        })
    
    def get_projects(self, project_ids, *args):
        try:
            projects = self.pool.get('project.project').search(self.cr, self.uid, [('id','in', [project_ids])])
        except:
            projects = project_ids
        ret_list = []
        for project in projects:
            proj = self.pool.get('project.project').browse(self.cr, self.uid, project)
            ret_dict = {}
            ret_dict['id'] = proj.id
            ret_dict['name'] = proj.name
            if proj.child_id:
                child_ids = []
                for child in proj.child_id:
                    child_ids.append(child.id)
                ret_dict['child_ids'] = child_ids
            else:
                ret_dict['child_ids'] = False
            ret_list.append(ret_dict)
        return ret_list
    
    def get_tasks(self, project_id, *args):
            try:
                projects = self.pool.get('project.project').search(self.cr, self.uid, [('id','in', [project_id])])
            except:
                projects = project_id
            ret_list = []
            for project in projects:
                proj = self.pool.get('project.project').browse(self.cr, self.uid, project)
                for task in proj.tasks:
                    ret_dict = {}
                    ret_dict['name'] = task.name
                    ret_dict['description'] = task.description
                    ret_dict['date_deadline'] = task.date_deadline
                    ret_dict['user_id'] = task.user_id
                    ret_list.append(ret_dict)
            return ret_list
    def get_project_workers(self, project_id, *args):
        ret_list = []
        proj = self.pool.get('project.project').browse(self.cr, self.uid, project_id)
        user_list = []        
        for task in proj.tasks:
           ret_dict = {}
           if task.user_id.id not in user_list:
                user_list.append(task.user_id.id)
                ret_dict['user_id'] = task.user_id           
                ret_list.append(ret_dict)                    
        return ret_list
        
    def get_tasks_by_worker(self, project_id, user_id, *args):
        ret_list = []
        tasks_ids = self.pool.get('project.task').search(self.cr, self.uid, [('project_id','=',project_id),('user_id','=',user_id)])
        for task_id in tasks_ids:
            task = self.pool.get('project.task').browse(self.cr, self.uid, task_id)
            ret_dict = {}
            ret_dict['name'] = task.name
            ret_dict['description'] = task.description           
            ret_list.append(ret_dict)
        return ret_list
    
    def get_conditions(self, conditions_id, *args):
        conditions = self.pool.get('contract.conditions').browse(self.cr, self.uid, conditions_id)
        ret_list = []
        for condition in conditions.paragraph_ids:
            ret_dict = {}
            ret_dict['title'] = condition.name
            ret_dict['text'] = condition.text
            ret_list.append(ret_dict)
        return ret_list
        
    def get_services(self, *args):
        services_ids = self.pool.get('contract.services').search(self.cr, self.uid, [])
        ret_list = []
        for service in services_ids:
            ret_dict = {}
            service_data = self.pool.get('contract.services').browse(self.cr, self.uid, service)
            ret_dict['name'] = service_data.name
            ret_dict['details'] = service_data.details
            ret_list.append(ret_dict)
        if ret_list == []:
            ret_list = False
        return ret_list            

report_sxw.report_sxw('report.report_contract','sale.order','addons/report_contract/report/contract.rml',parser=report_contract)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

