# -*- encoding: utf-8 -*-
##############################################################################
#
#    Copyright (C) 2009 EduSense BV, Pieter J. Kersten, http://www.edusense.nl.
#    All Rights Reserved
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import os, stat
from reportlab.pdfbase import pdfmetrics
from reportlab.pdfbase import ttfonts
from reportlab.lib import fonts
import netsvc

__all__ = ['loadFonts']

def rl_isreg(filename, dirname):
    try:
        st = os.lstat(os.path.join(dirname, filename)) # Don't follow links
    except OSError, reason:
        if reason.errno == 2:
            return False
        raise
    return stat.S_ISREG(st.st_mode)

def loadFonts():
    from reportlab import rl_config
    rl_config.warnOnMissingFontGlyphs = 0
    logger = netsvc.Logger()

    for dirname in rl_config.TTFSearchPath:
        for filename in [x for x in os.listdir(dirname)
                         if x.lower().endswith('.ttf') and rl_isreg(x, dirname)
                        ]:
            try:
                face = ttfonts.TTFontFace(filename)
                face.extractInfo(0) # Only header is needed
            except ttfonts.TTFError, reason:
                logger.notifyChannel(
                    'init', netsvc.LOG_WARNING,
                    'module report_truetype: registration of font file %s failed: %s' % (
                        filename, reason.message
                    ))
            else:
                logger.notifyChannel(
                    'init', netsvc.LOG_INFO,
                    'module report_truetype: registering font %s (%s)' % (
                        face.name, filename
                    ))
                font = ttfonts.TTFont(face.name, filename)
                pdfmetrics.registerFont(font)
                if not font._multiByte:
                    # Already done in registerFont with multi-byte fonts
                    ttname = font.fontName.lower()
                    fonts.addMapping(ttname, 0, 0, font.fontName)
                    fonts.addMapping(ttname, 1, 0, font.fontName)
                    fonts.addMapping(ttname, 0, 1, font.fontName)
                    fonts.addMapping(ttname, 1, 1, font.fontName)

loadFonts()
