# -*- encoding: utf-8 -*-
##############################################################################
#
#    Copyright (C) 2009 Numérigraphe SARL.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import wizard
import pooler
import tools
import netsvc

# Form for the 1st step
step1_form = '''
<form string="Global Rebate Line">
    <field name="product_id" colspan="4"/>
    <field name="percent" colspan="4" />
</form>
'''
# Fields for the 1st step
step1_fields = {
    'product_id': {
        'string': 'Rebate Product',
        'help': "The product representing the rebate. It should be configured as a service, make-to-stock.",
        'type': 'many2one',
        'relation': 'product.product',
        'required': True,
    },
    'percent': {
        'string': 'Rebate %',
        'help': 'The percent amount of rebate',
        'type': 'float',
        'required': True,
    },
}

def _add_rebate_line(self, cr, uid, data, context):
    """
    Add a sale order line with a global rebate.
    """
    pool = pooler.get_pool(cr.dbname)
    obj_order = pool.get('sale.order')
    obj_line = pool.get('sale.order.line')
    obj_product = pool.get('product.product')

    orders = obj_order.browse(cr, uid, data['ids'], context)
    product = obj_product.browse(cr, uid, data['form']['product_id'], context)
    for order in orders:
        # Only act on draft orders
        if order.state != 'draft':
            raise wizard.except_wizard(_('Cannot add line'), _('Rebate lines can only be added to draft orders!'))

        amount_per_tax = {}
        # Collect the amounts and taxes
        for line in order.order_line:
            if line.tax_id:
                if len(line.tax_id) > 1:
                    raise wizard.except_wizard(_('Too many taxes'), _('The amount of the rebate line can not be calculated if the order line is subjected to more than one tax!'))
                amount_per_tax[line.tax_id[0].id] = amount_per_tax.get(line.tax_id[0].id, 0.0) + line.price_subtotal
            else:
                amount_per_tax[False] = amount_per_tax.get(False, 0.0) + line.price_subtotal

        # Add the rebate line
        for (tax, amount) in amount_per_tax.iteritems():
            obj_line.create(cr, uid, {
                'order_id': order.id,
                'name': "%s (%.2f%%)" % (product.name, data['form']['percent']),
                'product_uom_qty': 1,
                'product_uom': product.uom_id.id,
                'product_id': data['form']['product_id'],
                'price_unit':-(amount * data['form']['percent']) / 100.0,
                'tax_id': tax and [(6, 0, [tax])],
                'type': 'make_to_stock',
                'state': order.state,
                'sequence': 0,
            })
    return {}

class wizard_add_rebate_line(wizard.interface):
    states = {
        'init': {
            'actions':[],
            'result': {
                'type': 'form',
                'arch': step1_form,
                'fields': step1_fields,
                'state':[('end', 'Cancel', 'gtk-cancel'), ('add', 'Add Rebate Line', 'gtk-go-forward')]
            }
        },
        'add': {
            'actions':[_add_rebate_line],
            'result': {
                'type': 'state',
                'state':'end',
            }
        },
    }
wizard_add_rebate_line('sale_rebate_line.add')
