# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution    
#    Copyright (C) 2009 Smile.fr. All Rights Reserved
#    authors: Raphaël Valyi, Xavier Fernandez
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import fields, osv

from mx import DateTime
import netsvc


class sale_order(osv.osv):
    _inherit = "sale.order"
    
    def copy(self, cr, uid, id, default=None, context=None):
        if default is None:
            default = {}
        default = default.copy()
        default.update({'payment_ids':False,})
        return super(sale_order, self).copy(cr, uid, id, default, context)
    
    def _get_customer_name(self, cr, uid, ids, field_name, arg, context):
        res = {}
        for so in self.browse(cr, uid, ids):
            res[so.id] = so.partner_id.name
        return res
    
    def _set_customer_name(self, cr, user, id, name, value, arg, context):
        pass
    
    def _amount_paid(self, cr, uid, ids, field_name, arg, context):
        res = {}
        for so in self.browse(cr, uid, ids):
            total = 0
            for payment in so.payment_ids:
                total += payment.debit - payment.credit
            res[so.id] = total
        return res
    
    _columns = {
                'customer_name':fields.function(_get_customer_name, type='char', fnct_inv=_set_customer_name, method=True, string='New Customer Name', size=64),
            #TODO: use it to reset the shop_id.warehouse_id used in original action_ship_create
                'amount_paid': fields.function(_amount_paid, string="Deposits", method=True, type="float"),
                'default_customer': fields.boolean("Anonymous sale order", readonly=True, states={'draft':[('readonly', False)]}),
                'payment_ids': fields.one2many('account.move.line', 'sale_id', 'Deposits', readonly=True, states={'prevalidated':[('readonly', False)], 'progress':[('readonly', False)]}),
                'state': fields.selection([
            ('draft', 'Quotation'),
            ('waiting_date', 'Waiting Schedule'),
            ('manual', 'Manual In Progress'),
            ('progress', 'In Progress'),
            ('shipping_except', 'Shipping Exception'),
            ('invoice_except', 'Invoice Exception'),
            ('done', 'Done'),
            ('cancel', 'Cancel'),
            ('prevalidated', 'Prevalidated'),
            ], 'Order State', readonly=True, help="Gives the state of the quotation or sale order. The exception state is automatically set when a cancel operation occurs in the invoice validation (Invoice Exception) or in the packing list process (Shipping Exception). The 'Waiting Schedule' state is set when the invoice is confirmed but waiting for the scheduler to run on the date 'Date Ordered'.", select=True),
        }

    _defaults = {
                 'default_customer': lambda * a:False,
                 }

    
    def onchange_customer_name(self, cr, uid, ids, name, context={}):
        partner_ids = self.pool.get('res.partner').search(cr, uid, [('name', 'ilike', name)])
        if len(partner_ids) > 0:
            names = self.pool.get('res.partner').read(cr, uid, partner_ids, ['name'])
            names = [i['name'] for i in names]
            names = "\n".join(names)
            return {'value':{'warning_showed': True}, 'warning': {'title':"Warning", 'message':"A partner with a similar name already exists:\n%s" % names}}
        return {}
    
    def default_get(self, cr, uid, fields_list, context=None):
        value = super(sale_order, self).default_get(cr, uid, fields_list, context)
        if 'partner_id' in fields_list:
            
            default_customer_id = self.pool.get('res.partner').get_default_customer_id(cr, uid)
            value['partner_id'] = default_customer_id            
        return value
    
    def _update_vals_with_new_customer(self, cr, uid, vals, default_customer_id):
        new_customer_id = self.pool.get('res.partner').create(cr, uid, {'name': vals['customer_name'], 'lang': self.pool.get('res.users').browse(cr, uid, uid).context_lang})
        vals['partner_id'] = new_customer_id
        new_address_id = self.pool.get('res.partner.address').create(cr, uid, {'name': "Undefined address", 'partner_id': new_customer_id})
        vals['partner_order_id'] = new_address_id
        vals['partner_invoice_id'] = new_address_id
        vals['partner_shipping_id'] = new_address_id
        return vals

    def create(self, cr, uid, vals, context={}):
        context['from_sale'] = True #used to make sure payments moves will be properly balanced
        default_customer_id = self.pool.get('res.partner').get_default_customer_id(cr, uid)
        if default_customer_id:
            default_customer_name = self.pool.get('res.partner').read(cr, uid, default_customer_id, ['name'])['name']
            if vals['partner_id'] == default_customer_id:
                if vals.get('customer_name', False) and vals.get('customer_name', False) != default_customer_name:
                    vals = self._update_vals_with_new_customer(cr, uid, vals, default_customer_id)
                if vals['partner_id'] == default_customer_id and vals['default_customer'] == False:
                    raise osv.except_osv(_('Error !'), _("Sale order is supposed to be non-anonymous but the client is."))
        created_id = super(sale_order, self).create(cr, uid, vals, context)
        return created_id

    def write(self, cr, uid, ids, vals, context={}):
        context['from_sale'] = True #used to make sure payments moves will be properly balanced
        if isinstance(ids, list):
            for id in ids:
                self.write(cr, uid, id, vals, context)
            return True
        else:
            default_customer_id = self.pool.get('res.partner').get_default_customer_id(cr, uid)
            if default_customer_id:
                so_partner_id = self.read(cr, uid, ids, ['partner_id'])['partner_id'][0]
                if so_partner_id == default_customer_id:
                    if 'customer_name' in vals:
                        vals = self._update_vals_with_new_customer(cr, uid, vals, default_customer_id)
            return super(sale_order, self).write(cr, uid, ids, vals, context)
        
    def unlink(self, cr, uid, ids, context={}):
        context['from_sale'] = True #used to make sure payments moves will be properly balanced
        return super(sale_order, self).unlink(cr, uid, ids, context)
    
    def _make_invoice(self, cr, uid, order, lines, context={}):
        """take care of the company id, especially because the user generating the invoice might be different form the one encoding the sale order"""
        invoice_id = super(sale_order, self)._make_invoice(cr, uid, order, lines, context)
        self.pool.get('account.invoice').write(cr, uid, invoice_id, {'company_id': order.user_id.company_id.id})
        
        #journal selection: a bit like account.invoice._get_journal but also company aware!
        res = self.pool.get('account.journal').search(cr, uid, [('type', '=', 'sale'), ('company_id', '=', order.user_id.company_id.id)], limit=1)
        if res:
            self.pool.get('account.invoice').write(cr, uid, invoice_id, {'journal_id': res[0]}) 

        return invoice_id


    def customize_procurement(self, cr, uid, ids, *args):
        return True
    
    def post_deposits(self, cr, uid, ids, *args):
        """validate point of sale payments eventually"""
        for order in self.browse(cr, uid, ids, {}):
            if order.shop_id.deposit_validation == 'sale_confirm':
                for payment in order.payment_ids:
                    self.pool.get('account.move').post(cr, uid, [payment.move_id.id], *args)

    def action_ship_create(self, cr, uid, ids, *args):
        for order in self.browse(cr, uid, ids, {}):
            if (order.partner_id.id == self.pool.get('res.partner').get_default_customer_id(cr, uid) and order.amount_paid < order.amount_total):
                raise osv.except_osv(_('Error !'), _("Anonymous customer should pay 100% first!"))
        """Run the MRP when the order picking is created"""
        result = super(sale_order, self).action_ship_create(cr, uid, ids, *args)
        self.post_deposits(cr, uid, ids, *args)
        self.customize_procurement(cr, uid, ids, *args)
        cr.commit()
        self.pool.get('mrp.procurement').run_scheduler(cr, uid, automatic=False, use_new_cursor=cr.dbname, context={})
        return result
 


sale_order()


class sale_order_line(osv.osv):
    _inherit = "sale.order.line"
    
    def default_get(self, cr, uid, fields_list, context=None):
        value = super(sale_order_line, self).default_get(cr, uid, fields_list, context)
        print value
        if 'product_uom' in fields_list:
            value['product_uom'] = self.pool.get('product.uom').search(cr, uid, [('name', '=', 'PCE')])[0]
        print value    
        return value
    
sale_order_line()


class sale_shop(osv.osv):
    _inherit = "sale.shop"
    
    _columns = {
                    'default_deposit_percent': fields.integer("Default Deposit %", help="percentage of the deposits by default"),
                    'deposit_validation': fields.selection([('sale_confirm','Automatically once Order Confirmed'),('invoice_confirm','Automatically once Invoice is Confirmed'),('manual','Manual Validation Only')], 'Deposits Accounting Validation', required=True),
                }
    
    _defaults = {
                    'default_deposit_percent': lambda * a: 30,
                    'deposit_validation': lambda *a: 'invoice_confirm'
                 }
    
sale_shop()