# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution    
#    Copyright (C) 2009 Smile (<http://www.smile.fr>). All Rights Reserved
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import ir
from osv import fields, osv, orm
import netsvc
import pooler
from mx.DateTime import RelativeDateTime, now

class sartre_rule(osv.osv):
    _name = 'sartre.rule'
    _description = 'Sartre Rule'

    def _get_trigger_date_type(self, cr, uid, ids, name, args, context={}):
        res = {}
        if isinstance(ids, (int, long)):
            ids = [ids]
        for rule in self.browse(cr, uid, ids):
            if rule.trigger_date_type_display1 == 'other_date':
                res[rule.id] = rule.trigger_date_type_display2_id.name
            else:
                res[rule.id] = rule.trigger_date_type_display1
        return res

    def onchange_get_domain_force(self, cr, uid, ids, condition_ids, domain_force, context={}):
        """
        Build domain expression from conditions
        """
        res = {}
        domain_force = domain_force and eval(domain_force) or []
        for condition in self.pool.get('sartre.condition').read(cr, uid, condition_ids, ['domain']):
            domain_force += condition['domain'] and eval(condition['domain']) or []
        return res

    _columns = {
        'name': fields.char("Name", size=64, required=True),
        'model_id': fields.many2one('ir.model', 'Object', domain=[('trigger_rules_allowed', '=', True)], required=True, help=""),
        'active': fields.boolean("Active"),
        'trigger_create': fields.boolean("Trigger Creation"),
        'trigger_write': fields.boolean("Trigger Update"),
        'trigger_unlink': fields.boolean("Trigger Unlink"),
        'trigger_date': fields.boolean("Trigger Date"),
        'trigger_date_type': fields.function(_get_trigger_date_type, method=True, type='char', size=64, string='Trigger Date Type', store=True),
        'trigger_date_type_display1': fields.selection([('create_date', 'Creation Date'), ('write_date', 'Update Date'), ('other_date', 'Other Date')], 'Trigger Date Type 1', size=16),
        'trigger_date_type_display2_id': fields.many2one('ir.model.fields', 'Trigger Date Type 2', domain="[('ttype','in',['date','datetime']),('model_id','=',model_id)]"),
        'trigger_date_range': fields.integer('Delay'),
        'trigger_date_range_type': fields.selection([('minutes', 'Minutes'), ('hours', 'Hours'), ('work_days', 'Work Days'), ('days', 'Days'), ('weeks', 'Weeks'), ('months', 'Months')], 'Delay Type'),
        'trigger_date_range_operand': fields.selection([('after', 'After'), ('before', 'Before')], 'Delay Operand'),
        'interval_number': fields.integer('Interval Number'),
        'interval_type': fields.selection([('minutes', 'Minutes'), ('hours', 'Hours'), ('work_days', 'Work Days'), ('days', 'Days'), ('weeks', 'Weeks'), ('months', 'Months')], 'Interval Unit'),
        'nextcall': fields.datetime("Next Call"),
        'condition_ids': fields.one2many('sartre.condition', 'sartre_rule_id', "Conditions", help="The rule is satisfied if all conditions are True"),
        'domain_force': fields.char('Force Domain', size=250),
        'action_ids': fields.many2many('ir.actions.server', 'sartre_rule_server_action_rel', 'server_action_id', 'sartre_rule_id', "Actions"),
    }

    _defaults = {
        'active': lambda * a: True,
        'trigger_date_type_display1': lambda * a: 'create_date',
        'trigger_date_range': lambda * a: 2,
        'trigger_date_range_type': lambda * a: 'days',
        'trigger_date_range_operand': lambda * a: 'after',
        'interval_number': lambda * a: 1,
        'interval_type': lambda * a: 'hours',
        'nextcall': lambda * a: now().strftime("%Y-%m-%d %H:%M:%S"),
    }

    def run_now(self, cr, uid, ids, context={}):
        """
        Function to execute server actions now
        """
        today_datetime = now()
        today_str = today_datetime.strftime('%Y-%m-%d %H:%M:%S')
        for rule in self.browse(cr, uid, ids):
            try:
                domain = rule.domain_force and eval(rule.domain_force.replace('%today', today_str)) or []
                if not domain:
                    for condition in rule.condition_ids:
                        domain += eval(condition.domain.replace('%today', today_str))
                if rule.trigger_date:
                    interval_number = rule.trigger_date_range
                    interval_type = str(rule.trigger_date_range_type)
                    interval_operand = rule.trigger_date_range_operand
                    # Update rule next call
                    self.write(cr, uid, rule.id, {'nextcall': today_datetime + RelativeDateTime(**{interval_type:interval_number})}, context)
                    # Add datetime filter
                    field = rule.trigger_date_type
                    limit_date = today_datetime
                    if interval_operand == 'after':
                        limit_date -= RelativeDateTime(**{interval_type:interval_number})
                    if interval_operand == 'before':
                        limit_date += RelativeDateTime(**{interval_type:interval_number})
                    datetime_condition = (field, '<=', limit_date.strftime("%Y-%m-%d %H:%M:%S"))
                    domain.append(datetime_condition)
                model = self.pool.get(rule.model_id.model)
                if 'active_object_ids' in context and context['active_object_ids']:
                    # To avoid infinite recursion
                    if 'rules' in context and rule.id in context['rules'].keys():
                        context['active_object_ids'] = list(set(context['active_object_ids']) - set(context['rules'][rule.id]))
                    # Check if active objects respect all conditions based on old values
                    indexes = [domain.index(item) for item in domain if isinstance(item, tuple) and item[0].startswith('OLD_')]
                    if not indexes:
                        domain += [('id', 'in', context['active_object_ids'])]
                    else:
                        old_values = context.get('old_values', {})
                        domain_on_old_values = [domain[position] for position in indexes]
                        changing_fields = old_values and old_values[old_values.keys()[0]].keys() or []
                        domain_fields = [f for (f, o, v) in domain_on_old_values if f.replace('OLD_', '') in changing_fields]
                        if len(domain_fields) == len(indexes):
                            operators = {'=':'==', '<>':'!=', 'like':'in', 'ilike':'in', '^':'.startswith', '$':'.endswith', '~':'!='}
                            for index in indexes:
                                condition = domain[index]
                                # Replace filters on old values by filters on active objects
                                active_object_ids = context['active_object_ids']
    
                                prefix = ''
                                field = condition[0].replace('OLD_', '')
                                operator = condition[1]
                                if 'not ' in operator:
                                    prefix = 'not '
                                    operator = operator.replace('not ', '')
                                domain_old_value = condition[2]
                                if operator in ['like', 'ilike']:
                                    domain_old_value.replace('%', '')
                                if operator == '~':
                                    # TODO: read all fields in one time (for all 'is changing' conditions)
                                    current_values = dict([(item['id'], (item[field] and isinstance(item[field], tuple)) and (len(item[field]) == 1 and item[field][0] or item[field][1]) or item[field]) for item in model.read(cr, uid, active_object_ids, [field])])
                                for id in active_object_ids:
                                    object_old_value = old_values[id][field]
                                    if isinstance(object_old_value, unicode):
                                        object_old_value = str(object_old_value.encode('utf-8'))
                                    if operator == 'ilike':
                                        object_old_value = object_old_value.lower()
                                        domain_old_value = domain_old_value.lower()
                                    if operator == '~':
                                        domain_old_value = current_values[id]
                                    object_old_value = (isinstance(object_old_value, str) and "'%s'" or "%s") % object_old_value
                                    operator = operator in operators.keys() and operators[operator] or operator
                                    if isinstance(domain_old_value, unicode):
                                        domain_old_value = str(domain_old_value.encode('utf-8'))
                                    domain_old_value = (isinstance(domain_old_value, str) and "'%s'" or "%s") % domain_old_value
                                    validated = eval(prefix + object_old_value + operator + domain_old_value)
                                    if not validated:
                                        active_object_ids.remove(id)
                                domain[index] = ('id', 'in', active_object_ids)
            except Exception, e:
                self.pool.get('sartre.exception').create(cr, uid, {'rule_id': rule.id, 'exception_type': 'condition', 'exception': e})
                pass
            if 'OLD_' not in str(domain) and rule.model_id:
                # Search objects which validate rule conditions
                rule_object_ids = model.search(cr, uid, domain)
                # Execute server actions
                if rule_object_ids:
                    for object_id in rule_object_ids:
                        context['active_id'] = object_id
                        for action in rule.action_ids:
                            try:
                                self.pool.get('ir.actions.server').run(cr, uid, [action.id], context)
                            except Exception, e:
                                self.pool.get('sartre.exception').create(cr, uid, {'rule_id': rule.id, 'exception_type': 'action', 'res_id': object_id, 'action_id': action.id, 'exception': e})
                                continue
                    if not 'rules' in context:
                        context['rules'] = {}
                    context['rules'].setdefault(rule.id, [])
                    context['rules'][rule.id] += rule_object_ids

    def check_rules(self, cr, uid, context={}):
        """
        Function called by the scheduler to check date based trigger rules
        """
        # Search rules to execute
        rule_ids = self.search(cr, uid, [('active', '=', True), ('trigger_date', '=', True), ('nextcall', '<=', now().strftime("%Y-%m-%d %H:%M:%S"))])
        if rule_ids:
            # Launch rules execution
            self.run_now(cr, uid, rule_ids, context)

sartre_rule()

class sartre_condition(osv.osv):
    _name = 'sartre.condition'
    _description = 'Sartre Condition'
    _rec_name = 'field_id'

    def onchange_get_domain(self, cr, uid, ids, field_id=False, operator=False, opposite=False, value=False, value_age='current', context={}):
        """
        Build domain expression from condition items
        """
        res = {}
        if field_id and operator and value:
            field = self.pool.get('ir.model.fields').browse(cr, uid, field_id)
            field_name = ((value_age == 'old' or operator == '~') and 'OLD_' or '') + field.name + (field.ttype == 'many2one' and '.' + self.pool.get(field.relation)._rec_name or '')
            if operator == 'in':
                value = value.split(',')
            if operator in ['ilike', 'like']:
                value = '%' + str(value).replace('%', '') + '%'
            if operator in ['^', '$'] and value_age == 'current':
                operator = 'like'
                value = (operator != '^' and '%' or '') + str(value).replace('%', '') + (operator != '$' and '%' or '')
            if operator == '~':
                value = ''
            if opposite:
                opposites = {'=':'<>', '<=':'>', '>=':'<', 'like': 'not like', 'ilike': 'not ilike', '^': 'not ^', '$': 'not $', '~': 'not ~', 'in': 'not in', 'child_of': 'not child_of'}
                operator = opposites[operator]
            res['value'] = {'domain': str([(field_name, operator, value)])}
        return res

    _columns = {
        "sartre_rule_id": fields.many2one('sartre.rule', "Rule", required=True, ondelete='cascade'),
        "field_id": fields.many2one('ir.model.fields', "Field", domain="[('model_id','=',parent.model_id),('ttype','not in',['one2many','many2many'])]"),
        "operator": fields.selection([('=', 'Equals'), ('<=', 'Lesser than'), ('>=', 'Greater than'), ('like', 'Contains'), ('ilike', 'Contains Exactly'), ('^', 'Starts With'), ('$', 'Ends With'), ('~', 'Is Changing'), ('in', 'Belongs To'), ('child_of', 'Child Of')], "Operator"),
        "opposite": fields.boolean('Opposite'),
        "value": fields.char("Value", size=128),
        "value_age": fields.selection([
            ('current', 'Current Value'),
            ('old', 'Old Value'),
             ], "Value Age", select=True),
        "domain": fields.char("Domain", size=256, required=True),
    }

    _defaults = {
        'value_age': lambda * a: 'current',
    }

sartre_condition()

class sartre_exception(osv.osv):
    _name = 'sartre.exception'
    _description = 'Sartre Exception'
    _rec_name = 'rule_id'
   
    _columns = {
        'rule_id': fields.many2one('sartre.rule', 'Rule', required=False, select=True),
        'exception_type': fields.selection([
            ('condition','Condition'),
            ('action','Action'),
             ], 'Type', required=False, select=True),
        'res_id': fields.integer('Resource', required=False),
        'action_id': fields.many2one('ir.actions.server', 'Action', required=False, select=True),
        'exception': fields.text('Exception', required=False),
        'create_date': fields.datetime('Creation Date'),
    }

sartre_exception()

def check_method_based_trigger_rules(self, cr, uid, method, context={}):
    """
    Function to check method based trigger rules
    """
    rule_obj = self.pool.get('sartre.rule')
    if rule_obj:
        # Search rules to execute
        model_ids = self.pool.get('ir.model').search(cr, uid, [('model', '=', self._name)])
        rule_ids = rule_obj.search(cr, uid, [('model_id', 'in', model_ids), ('active', '=', True), ('trigger_' + method, '=', True)])
        if rule_ids:
            # Launch rules execution
            rule_obj.run_now(cr, uid, rule_ids, context)

def create_object_and_check_rules(self, cr, uid, vals, context={}):
    """
    Override create method to check create method based trigger rules
    """
    result = orm.orm.create(self, cr, uid, vals, context)
    if result:
        context_copy = context and context.copy() or {}
        context_copy['active_object_ids'] = [result]
        check_method_based_trigger_rules(self, cr, uid, 'create', context_copy)
    return result

def write_object_and_check_rules(self, cr, uid, ids, vals, context={}):
    """
    Override write method to check write method based trigger rules
    """
    if isinstance(ids, (int, long)):
        ids = [ids]
    context_copy = context and context.copy() or {}
    context_copy['active_object_ids'] = ids
    context_copy['old_values'] = dict([(item['id'], item) for item in self.read(cr, uid, ids)])
    result = orm.orm.write(self, cr, uid, ids, vals, context)
    if result:
        check_method_based_trigger_rules(self, cr, uid, 'write', context_copy)
    return result

def unlink_object_and_check_rules(self, cr, uid, ids, context={}):
    """
    Override unlink method to check unlink method based trigger rules
    """
    if isinstance(ids, (int, long)):
        ids = [ids]
    context_copy = context and context.copy() or {}
    context_copy['active_object_ids'] = ids
    context_copy['old_values'] = dict([(item['id'], item) for item in self.read(cr, uid, ids)])
    result = orm.orm.unlink(self, cr, uid, ids, context)
    if result:
        check_method_based_trigger_rules(self, cr, uid, 'unlink', context_copy)
    return result

osv.osv.create = create_object_and_check_rules
osv.osv.write = write_object_and_check_rules
osv.osv.unlink = unlink_object_and_check_rules
