#!/usr/bin/env python
#coding: utf-8

# (c) 2007 Sednacom <http://www.sednacom.fr>
# author : gael@sednacom.fr

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from osv import osv, fields
import tools
from mx import DateTime as mdt
import time
import netsvc
import ir





class stock_period(osv.osv):
	_name = "stock.period"
	_description = "stock period"

	_columns = {
		'name' : fields.char('Name', size=64, required=True, readonly=True),
		'stock_period_line' : fields.one2many('stock.period.line', 'stock_period_id', 'Stock Lines', select=True, readonly=True),
		'date_start' : fields.date('Date start', method=True, type='date', readonly=True),
		'date_end' : fields.date('Date end', method=True, type='date', readonly=True),
		'type' : fields.selection([
			('day','Day'),
			('week','Week'),
			('month','Month'),
			('Year','Year'),
			], 'Type', readonly=True, select=True),
		'during' : fields.char('During', size=64, required=True, readonly=True),
		'state': fields.selection([
			('open','Open'),
			('close','Close'),
			], 'Period State', readonly=True, select=True), 
		}

	_defaults = {
	'type' : lambda *a: 'month',
	'during' : lambda *a : '1',
	'state': lambda *a: 'open',
	}

	_order ="date_start desc, date_end desc"

	def unlink(self, cr, uid, ids):
		raise osv.except_osv('Invalid action !', 'Cannot delete a stock period')
		osv.osv.unlink(self, cr, uid, unlink_ids)
		return True


	def _create_period(self, cr, uid, date_start, date_end, name, during, type, context={}):
		crit=[('state','=','open')]
		test=self.search(cr, uid, crit)
		test = len(test)
		crit2=[('active','=',True)]
		max_period_id = self.pool.get('stock.period.nb.open').search(cr, uid, crit2)
		max_period = self.pool.get('stock.period.nb.open').browse(cr, uid, max_period_id)[0].number
		print test," < ",max_period
		if test < max_period:
			#create the table for stock for a period
			product_obj = self.pool.get('product.product')
			#print "--get pord_ids"
			location_obj = self.pool.get('stock.location')
			#print "--get location_ids"
			ids = location_obj.search(cr, uid, [('usage','=','internal')])
			#print "--search ids"
			product_ids = product_obj.search(cr, uid, [])
			res = {'name':name,'date_start':date_start,'date_end':date_end,'stock_period_line':[],'during':during,'type':type,'state':'open'}
			#for prod in product_ids:
				#print '----parcourt du produit ',prod
				#print '------location ',loc
			for loc in ids:
				prods = self._product_get_multi_location(cr, uid,loc,product_ids,date_start,date_end)
				#print 'fin product get',prods
				for test in prods:
					res2={'product_id':test,'quantity':prods[test],'location_id':loc}
					res['stock_period_line'].append((0,0,res2))

#				for loc in ids:
#					#print '------location ',loc
#					test = self._product_get_multi_location(cr, uid,loc,[prod],date_start,date_end)
#					#print 'fin product get',test
#					qty = test.values()
#					#if qty[0] != 0:
#					res2={'product_id':prod,'quantity':qty[0],'location_id':loc}
#					res['stock_period_line'].append((0,0,res2))
			result = self.pool.get('stock.period').create(cr, uid, res)
			return result
		else:
			raise osv.except_osv('Couldn\'t create the period','The maximum of opened periods have been done, please close one or any opened periods.')

	def close_period(self, cr, uid, ids, context={}):
		result = self.pool.get('stock.period').write(cr, uid, ids,{'state':'close'})
		return result

	def _product_get_multi_location(self, cr, uid, ids, product_ids=False, date_start=False, date_end=False, context={}, states=['done'], what=('in', 'out')):
		ids = ids and [ids] or []
		states_str = ','.join(map(lambda s: "'%s'" % s, states))
		if not product_ids:
			product_ids = self.pool.get('product.product').search(cr, uid, [])
		res = {}
		for id in product_ids:
			res[id] = 0.0
		if not ids:
			return res
			
		prod_ids_str = ','.join(map(str, product_ids))
		location_ids_str = ','.join(map(str, ids))
		results = []
		results2 = []
		if 'in' in what:
			# all moves from a location out of the set to a location in the set
			cr.execute(
				'select sum(product_qty), product_id, product_uom '\
				'from stock_move '\
				'where location_id not in ('+location_ids_str+') '\
				'and location_dest_id in ('+location_ids_str+') '\
				'and product_id in ('+prod_ids_str+') '\
				'and state in ('+states_str+') '\
				'and date_planned between \''+date_start+'\' and \''+date_end+'\' '\
				'group by product_id,product_uom'
			)
			results = cr.fetchall()
		if 'out' in what:
			# all moves from a location in the set to a location out of the set
			cr.execute(
				'select sum(product_qty), product_id, product_uom '\
				'from stock_move '\
				'where location_id in ('+location_ids_str+') '\
				'and location_dest_id not in ('+location_ids_str+') '\
				'and product_id in ('+prod_ids_str+') '\
				'and state in ('+states_str+') '\
				'and date_planned between \''+date_start+'\' and \''+date_end+'\' '\
				'group by product_id,product_uom'
			)
			results2 = cr.fetchall()
		uom_obj = self.pool.get('product.uom')
		for amount, prod_id, prod_uom in results:
			amount = uom_obj._compute_qty(cr, uid, prod_uom, amount,
					context.get('uom', False))
			res[prod_id] += amount
		for amount, prod_id, prod_uom in results2:
			amount = uom_obj._compute_qty(cr, uid, prod_uom, amount,
					context.get('uom', False))
			res[prod_id] -= amount
		return res

stock_period()

class stock_period_line(osv.osv):
	_name = "stock.period.line"
	_description = "stock period line"

	_columns = {
		'stock_period_id' : fields.many2one('stock.period', 'Stock period',  ondelete='cascade', select=True, readonly=True),
		'product_id' : fields.many2one('product.product', 'Product', required=True, readonly=True),
		'location_id' : fields.many2one('stock.location', 'Location', required=True, readonly=True),
		'quantity' : fields.float('Quantity', required=True, readonly=True),
		}
	_rec_name = "product_id"

stock_period_line()

class stock_picking(osv.osv):
	_name = "stock.picking"
	_description = "Packing list with stock period"
	_inherit = "stock.picking"

	def check_date(self, cr, uid, date, context):
		date = context
		if date:
			crit = [('date_start','<=',date),('date_end','>=',date),('state','=','open')]
			res = self.pool.get('stock.period').search(cr, uid, crit)
			if not res:
				raise osv.except_osv('Could not validate the date','The date is not in an open stock period, please close the opened period and create a new period')
			else :
				return {} 

	def action_confirm(self, cr, uid, ids, *args):
		res= super(stock_picking,self).action_confirm(cr, uid, ids, *args)
		for pick in self.browse(cr,uid,ids):
			crit = [('date_start','<=',pick.date),('date_end','>=',pick.date),('state','=','open')]
			res2 = self.pool.get('stock.period').search(cr, uid, crit)
			if not res2:
				raise osv.except_osv('Could not Confirm the picking','The date is not in an open stock period, please close the opened period and create a new period')
		return res

	def action_assign(self, cr, uid, ids, *args):
		res= super(stock_picking,self).action_assign(cr, uid, ids, *args)
		for pick in self.browse(cr,uid,ids):
			crit = [('date_start','<=',pick.date),('date_end','>=',pick.date),('state','=','open')]
			res2 = self.pool.get('stock.period').search(cr, uid, crit)
			if not res2:
				raise osv.except_osv('Could not Assign the picking','The date is not in an open stock period, please close the opened period and create a new period')
		return res

	def force_assign(self, cr, uid, ids, *args):
		res= super(stock_picking,self).force_assign(cr, uid, ids, *args)
		for pick in self.browse(cr,uid,ids):
			crit = [('date_start','<=',pick.date),('date_end','>=',pick.date),('state','=','open')]
			res2 = self.pool.get('stock.period').search(cr, uid, crit)
			if not res2:
				raise osv.except_osv('Could not Force Assign the picking','The date is not in an open stock period, please close the opened period and create a new period')
		return res

	def test_finnished(self, cr, uid, ids):
		res= super(stock_picking,self).test_finnished(cr,uid,ids)
		for pick in self.browse(cr,uid,ids):
			crit = [('date_start','<=',pick.date),('date_end','>=',pick.date),('state','=','open')]
			res2 = self.pool.get('stock.period').search(cr, uid, crit)
			if not res2:
				raise osv.except_osv('Could not Finish the picking','The date is not in an open stock period, please close the opened period and create a new period')
		return res

#stock_picking()

class stock_move(osv.osv):
	_name = "stock.move"
	_description = "move with stock period"
	_inherit = "stock.move"

	def check_date(self, cr, uid, date, context):
		date = context
		if date:
			crit = [('date_start','<=',date),('date_end','>=',date),('state','=','open')]
			res2 = self.pool.get('stock.period').search(cr, uid, crit)
			if not res2:
				cr.execute("SELECT max(date_start),max(date_end) FROM stock_period WHERE state='open'")
				for sql in cr.fetchall():
					tmp=sql[0]
				if not tmp:
					raise osv.except_osv('Could not validate the date','you don\'t have an open period')
				else:
					raise osv.except_osv('Could not validate the date','The date is not in an open stock period, please choose another date between %s and %s' % (sql[0],sql[1]))
			else:
				# catch the product name in the move
				#prod_move = self.browse(cr,uid,ids)[0].product_id
				prod_move = self.browse(cr,uid,res2)[0].product_id
				# search if the catched product have been sale after the date of the move
				#crit = [('sale_line_id','!=','null'),('date','>=',move.date),('product_id','=',prod_move.id),('state','=','done')]
				crit = [('sale_line_id','!=','null'),('date_planned','>=',move.date_planned),('product_id','=',prod_move.id),('state','=','done')]
				sm_ids = self.pool.get('stock.move').search(cr, uid, crit)
				sm = self.pool.get('stock.move').browse(cr, uid, sm_ids)
				for sm_line in sm:
					prod = sm_line.sale_line_id.product_id
					#if prod.name == prod_move.name:
					if prod.id == prod_move.id:
						raise osv.except_osv('Could not validate the date','The product %s has been sold on %s, please choose a date after' % (prod_move.name,sm_line.date_planned))


	def action_confirm(self, cr, uid, ids, context={}):
		res= super(stock_move,self).action_confirm(cr, uid, ids, context)
		for move in self.browse(cr, uid, ids):
			#crit = [('date_start','<=',move.date),('date_end','>=',move.date),('state','=','open')]
			crit = [('date_start','<=',move.date_planned),('date_end','>=',move.date_planned),('state','=','open')]
			res2 = self.pool.get('stock.period').search(cr, uid, crit)
			if not res2:
				cr.execute("SELECT max(date_start),max(date_end) FROM stock_period WHERE state='open'")
				for sql in cr.fetchall():
					tmp=sql[0]
				if not tmp:
					raise osv.except_osv('Could not valid the date','you don\'t have an open period')
				else:
					raise osv.except_osv('Could not Confirm the move','The date is not in an open stock period, please choose another date between %s and %s' % (sql[0],sql[1]))
			else:
				# catch the product name in the move
				prod_move = self.browse(cr,uid,ids)[0].product_id
				# search if the catched product have been sale after the date of the move
				#crit = [('sale_line_id','!=','null'),('date','>=',move.date),('product_id','=',prod_move.id),('state','=','done')]
				crit = [('sale_line_id','!=','null'),('date_planned','>',move.date_planned),('product_id','=',prod_move.id),('state','=','done')]
				sm_ids = self.pool.get('stock.move').search(cr, uid, crit)
				sm = self.pool.get('stock.move').browse(cr, uid, sm_ids)
				for sm_line in sm:
					if sm_line.id in ids:
						continue
					prod = sm_line.sale_line_id.product_id
					#if prod.name == prod_move.name:
					if prod.id == prod_move.id:
						raise osv.except_osv('Could not Confirm the move','The product %s has been sold on %s, please choose a date after' % (prod_move.name,sm_line.date_planned))
		return res

	def action_assign(self, cr, uid, ids, *args):
		res= super(stock_move,self).action_assign(cr, uid, ids, *args)
		for move in self.browse(cr, uid, ids):
			#crit = [('date_start','<=',move.date),('date_end','>=',move.date),('state','=','open')]
			crit = [('date_start','<=',move.date_planned),('date_end','>',move.date_planned),('state','=','open')]
			res2 = self.pool.get('stock.period').search(cr, uid, crit)
			if not res2:
				cr.execute("SELECT max(date_start),max(date_end) FROM stock_period WHERE state='open'")
				for sql in cr.fetchall():
					tmp=sql[0]
				if not tmp:
					raise osv.except_osv('Could not valid the date','you don\'t have an open period')
				else:
					raise osv.except_osv('Could not Assign the move','The date is not in an open stock period, please choose another date between %s and %s' % (sql[0],sql[1]))
			else:
				# catch the product name in the move
				prod_move = self.browse(cr,uid,ids)[0].product_id
				# search if the catched product have been sale after the date of the move
				#crit = [('sale_line_id','!=','null'),('date','>=',move.date),('product_id','=',prod_move.id),('state','=','done')]
				crit = [('sale_line_id','!=','null'),('date_planned','>',move.date_planned),('product_id','=',prod_move.id),('state','=','done')]
				sm_ids = self.pool.get('stock.move').search(cr, uid, crit)
				sm = self.pool.get('stock.move').browse(cr, uid, sm_ids)
				for sm_line in sm:
					if sm_line.id in ids:
						continue
					prod = sm_line.sale_line_id.product_id
					#if prod.name == prod_move.name:
					if prod.id == prod_move.id:
						raise osv.except_osv('Could not Assign the move','The product %s has been sold on %s, please choose a date after' % (prod_move.name,sm_line.date_planned))
		return res

	def force_assign(self, cr, uid, ids, context={}):
		res= super(stock_move,self).force_assign(cr, uid, ids, context)
		for move in self.browse(cr, uid, ids):
			#crit = [('date_start','<=',move.date),('date_end','>=',move.date),('state','=','open')]
			crit = [('date_start','<=',move.date_planned),('date_end','>',move.date_planned),('state','=','open')]
			res2 = self.pool.get('stock.period').search(cr, uid, crit)
			if not res2:
				cr.execute("SELECT max(date_start),max(date_end) FROM stock_period WHERE state='open'")
				for sql in cr.fetchall():
					tmp=sql[0]
				if not tmp:
					raise osv.except_osv('Could not valid the date','you don\'t have an open period')
				else:
					raise osv.except_osv('Could not Force Assign the move','The date is not in an open stock period, please choose another date between %s and %s' % (sql[0],sql[1]))
		else:
				# catch the product name in the move
				prod_move = self.browse(cr,uid,ids)[0].product_id
				# search if the catched product have been sale after the date of the move
				#crit = [('sale_line_id','!=','null'),('date','>=',move.date),('product_id','=',prod_move.id),('state','=','done')]
				crit = [('sale_line_id','!=','null'),('date_planned','>',move.date_planned),('product_id','=',prod_move.id),('state','=','done')]
				sm_ids = self.pool.get('stock.move').search(cr, uid, crit)
				sm = self.pool.get('stock.move').browse(cr, uid, sm_ids)
				for sm_line in sm:
					if sm_line.id in ids:
						continue
					prod = sm_line.sale_line_id.product_id
					#if prod.name == prod_move.name:
					if prod.id == prod_move.id:
						raise osv.except_osv('Could not Force Assign the move','The product %s has been sold on %s, please choose a date after' % (prod_move.name,sm_line.date_planned))
		return res

	def action_done(self, cr, uid, ids, *args):
		res= super(stock_move,self).action_done(cr, uid, ids, *args)
		for move in self.browse(cr, uid, ids):
			#crit = [('date_start','<=',move.date),('date_end','>=',move.date),('state','=','open')]
			crit = [('date_start','<=',move.date_planned),('date_end','>',move.date_planned),('state','=','open')]
			res2 = self.pool.get('stock.period').search(cr, uid, crit)
			if not res2: 
				cr.execute("SELECT max(date_start),max(date_end) FROM stock_period WHERE state='open'")
				for sql in cr.fetchall():
					tmp=sql[0]
				if not tmp:
					raise osv.except_osv('Could not valid the date','you don\'t have an open period')
				else:
					raise osv.except_osv('Could not Finish the move','The date is not in an open stock period, please choose another date between %s and %s' % (sql[0],sql[1]))
			else:
				# catch the product name in the move
				prod_move = self.browse(cr,uid,ids)[0].product_id
				# search if the catched product have been sale after the date of the move
				#crit = [('sale_line_id','!=','null'),('date','>=',move.date),('product_id','=',prod_move.id),('state','=','done')]
				crit = [('sale_line_id','!=','null'),('date_planned','>',move.date_planned),('product_id','=',prod_move.id),('state','=','done')]
				sm_ids = self.pool.get('stock.move').search(cr, uid, crit)
				sm = self.pool.get('stock.move').browse(cr, uid, sm_ids)
				for sm_line in sm:
					if sm_line.id in ids:
						continue
					prod = sm_line.sale_line_id.product_id
					#if prod.name == prod_move.name:
					if prod.id == prod_move.id:
						raise osv.except_osv('Could not Finish the move','The product %s has been sold on %s, please choose a date after' % (prod_move.name,sm_line.date_planned))
stock_move()

class stock_period_nb_open(osv.osv):
	_name = "stock.period.nb.open"
	_description = "number of opened period"
	_columns = {
	'number' : fields.integer('Number of opened period',required=True),
	'active' : fields.boolean('Active',readonly=True)
	}
	_defaults = {
	'active': lambda *a : True,
	}
stock_period_nb_open()
